# -*- coding: utf-8 -*-


def get_axes(self):
    """Return axes list.
    Parameters
    ----------
    self: DataND
        a DataND object
    Returns
    -------
    axes_list
    """

    return self.axes
